@@ -19,9 +19,6 @@ class vec:
     pass
 
 def _create_vector_types():
-    name_to_dtype = {}
-    dtype_to_name = {}
-
     from pycuda.characterize import platform_bits
     if platform_bits() == 32:
         long_dtype = np.int32
@@ -32,19 +29,21 @@ def _create_vector_types():
 
     field_names = ["x", "y", "z", "w"]
 
+    from pycuda.tools import register_dtype
+
     for base_name, base_type, counts in [
         ('char', np.int8, [1,2,3,4]),
         ('uchar', np.uint8, [1,2,3,4]),
         ('short', np.int16, [1,2,3,4]),
         ('ushort', np.uint16, [1,2,3,4]),
-        ('int', np.uint32, [1,2,3,4]),
+        ('int', np.int32, [1,2,3,4]),
         ('uint', np.uint32, [1,2,3,4]),
         ('long', long_dtype, [1,2,3,4]),
         ('ulong', ulong_dtype, [1,2,3,4]),
         ('longlong', np.int64, [1,2]),
         ('ulonglong', np.uint64, [1,2]),
         ('float', np.float32, [1,2,3,4]),
-        ('ulonglong', np.float64, [1,2]),
+        ('double', np.float64, [1,2]),
         ]:
         for count in counts:
             name = "%s%d" % (base_name, count)
@@ -52,21 +51,17 @@ def _create_vector_types():
                 (field_names[i], base_type)
                 for i in range(count)])
 
-            name_to_dtype[name] = dtype
-            dtype_to_name[dtype] = name
+            register_dtype(dtype, name, alias_ok=True)
 
             setattr(vec, name, dtype)
 
             my_field_names = ",".join(field_names[:count])
-            setattr(vec, "make_"+name, 
+            setattr(vec, "make_"+name,
                     staticmethod(eval(
                         "lambda %s: array((%s), dtype=my_dtype)"
                         % (my_field_names, my_field_names),
                         dict(array=np.array, my_dtype=dtype))))
 
-    vec._dtype_to_c_name = dtype_to_name
-    vec._c_name_to_dtype = name_to_dtype
-
 _create_vector_types()
 
 # }}}
@@ -156,7 +151,7 @@ class GPUArray(object):
             for dim in shape:
                 s *= dim
         except TypeError:
-            assert isinstance(shape, int)
+            assert isinstance(shape, (int, long, np.integer))
             s = shape
             shape = (shape,)
 
@@ -632,6 +627,42 @@ class GPUArray(object):
 
         return result
 
+    def reshape(self, *shape):
+        # TODO: add more error-checking, perhaps
+        if isinstance(shape[0], tuple) or isinstance(shape[0], list):
+            shape = tuple(shape[0])
+        size = reduce(lambda x, y: x * y, shape, 1)
+        if size != self.size:
+            raise ValueError("total size of new array must be unchanged")
+
+        return GPUArray(
+                shape=shape,
+                dtype=self.dtype,
+                allocator=self.allocator,
+                base=self,
+                gpudata=int(self.gpudata))
+
+    def ravel(self):
+        return self.reshape(self.size)
+
+    def view(self, dtype=None):
+        if dtype is None:
+            dtype = self.dtype
+
+        old_itemsize = self.dtype.itemsize
+        itemsize = np.dtype(dtype).itemsize
+
+        if self.shape[-1] * old_itemsize % itemsize != 0:
+            raise ValueError("new type not compatible with array")
+
+        shape = self.shape[:-1] + (self.shape[-1] * old_itemsize // itemsize,)
+
+        return GPUArray(
+                shape=shape,
+                dtype=dtype,
+                allocator=self.allocator,
+                base=self,
+                gpudata=int(self.gpudata))
 
     # slicing -----------------------------------------------------------------
     def __getitem__(self, idx):
