@@ -63,26 +63,30 @@ source code with only those rights set forth herein.
 
 from pycuda.tools import context_dependent_memoize
 from pycuda.tools import dtype_to_ctype
-import numpy
+import numpy as np
 
 
 
 
 def get_reduction_module(out_type, block_size,
         neutral, reduce_expr, map_expr, arguments,
-        name="reduce_kernel", keep=False, options=[], preamble=""):
+        name="reduce_kernel", keep=False, options=None, preamble=""):
 
     from pycuda.compiler import SourceModule
     src = """
+        #include <pycuda-complex.hpp>
+
         #define BLOCK_SIZE %(block_size)d
         #define READ_AND_MAP(i) (%(map_expr)s)
         #define REDUCE(a, b) (%(reduce_expr)s)
 
-        typedef %(out_type)s out_type;
-
         %(preamble)s
 
-        __global__ void %(name)s(out_type *out, %(arguments)s,
+        typedef %(out_type)s out_type;
+
+        extern "C"
+        __global__
+        void %(name)s(out_type *out, %(arguments)s,
           unsigned int seq_count, unsigned int n)
         {
           __shared__ out_type sdata[BLOCK_SIZE];
@@ -123,7 +127,7 @@ def get_reduction_module(out_type, block_size,
           if (tid < 32)
           {
             // 'volatile' required according to Fermi compatibility guide 1.2.2
-            volatile out_type * smem = sdata;
+            volatile out_type *smem = sdata;
             if (BLOCK_SIZE >= 64) smem[tid] = REDUCE(smem[tid], smem[tid + 32]);
             if (BLOCK_SIZE >= 32) smem[tid] = REDUCE(smem[tid], smem[tid + 16]);
             if (BLOCK_SIZE >= 16) smem[tid] = REDUCE(smem[tid], smem[tid + 8]);
@@ -144,19 +148,31 @@ def get_reduction_module(out_type, block_size,
             "name": name,
             "preamble": preamble
             }
-    return SourceModule(src, options=options, keep=keep)
+    return SourceModule(src, options=options, keep=keep, no_extern_c=True)
 
 
 
 
-def get_reduction_kernel_and_types(out_type, block_size,
+def get_reduction_kernel_and_types(stage, out_type, block_size,
         neutral, reduce_expr, map_expr=None, arguments=None,
-        name="reduce_kernel", keep=False, options=[], preamble=""):
-    if map_expr is None:
-        map_expr = "in[i]"
+        name="reduce_kernel", keep=False, options=None, preamble=""):
 
-    if arguments is None:
-        arguments = "const %s *in" % out_type
+    if stage == 1:
+        if map_expr is None:
+            map_expr = "in[i]"
+
+    elif stage == 2:
+        if map_expr is None:
+            map_expr = "pycuda_reduction_inp[i]"
+
+        in_arg = "const %s *pycuda_reduction_inp" % out_type
+        if arguments:
+            arguments = in_arg + ", " + arguments
+        else:
+            arguments = in_arg
+
+    else:
+        assert False
 
     mod = get_reduction_module(out_type, block_size,
             neutral, reduce_expr, map_expr, arguments,
@@ -165,7 +181,7 @@ def get_reduction_kernel_and_types(out_type, block_size,
     from pycuda.tools import get_arg_type
     func = mod.get_function(name)
     arg_types = [get_arg_type(arg) for arg in arguments.split(",")]
-    func.prepare("P%sII" % "".join(arg_types), (block_size,1,1))
+    func.prepare("P%sII" % "".join(arg_types))
 
     return func, arg_types
 
@@ -175,14 +191,14 @@ def get_reduction_kernel_and_types(out_type, block_size,
 class ReductionKernel:
     def __init__(self, dtype_out,
             neutral, reduce_expr, map_expr=None, arguments=None,
-            name="reduce_kernel", keep=False, options=[], preamble=""):
+            name="reduce_kernel", keep=False, options=None, preamble=""):
 
         self.dtype_out = dtype_out
 
         self.block_size = 512
 
         s1_func, self.stage1_arg_types = get_reduction_kernel_and_types(
-                dtype_to_ctype(dtype_out), self.block_size,
+                1, dtype_to_ctype(dtype_out), self.block_size,
                 neutral, reduce_expr, map_expr,
                 arguments, name=name+"_stage1", keep=keep, options=options,
                 preamble=preamble)
@@ -190,8 +206,8 @@ class ReductionKernel:
 
         # stage 2 has only one input and no map expression
         s2_func, self.stage2_arg_types = get_reduction_kernel_and_types(
-                dtype_to_ctype(dtype_out), self.block_size,
-                neutral, reduce_expr,
+                2, dtype_to_ctype(dtype_out), self.block_size,
+                neutral, reduce_expr, arguments=arguments,
                 name=name+"_stage2", keep=keep, options=options,
                 preamble=preamble)
         self.stage2_func = s2_func.prepared_async_call
@@ -219,12 +235,18 @@ class ReductionKernel:
         f = s1_func
         arg_types = self.stage1_arg_types
 
+        stage1_args = args
+
         while True:
             invocation_args = []
             vectors = []
 
             for arg, arg_tp in zip(args, arg_types):
                 if arg_tp == "P":
+                    if not arg.flags.forc:
+                        raise RuntimeError("ReductionKernel cannot "
+                                "deal with non-contiguous arrays")
+
                     vectors.append(arg)
                     invocation_args.append(arg.gpudata)
                 else:
@@ -248,7 +270,7 @@ class ReductionKernel:
                 result = empty((block_count,), self.dtype_out, repr_vec.allocator)
 
             #print block_count, seq_count, self.block_size
-            f((block_count, 1), stream,
+            f((block_count, 1), (self.block_size, 1, 1), stream,
                     *([result.gpudata]+invocation_args+[seq_count, sz]))
 
             if block_count == 1:
@@ -256,7 +278,7 @@ class ReductionKernel:
             else:
                 f = s2_func
                 arg_types = self.stage2_arg_types
-                args = [result]
+                args = (result,) + stage1_args
 
 
 
@@ -324,8 +346,8 @@ def get_subset_dot_kernel(dtype_out, dtype_a=None, dtype_b=None):
 
 
 def get_minmax_neutral(what, dtype):
-    dtype = numpy.dtype(dtype)
-    if issubclass(dtype.type, numpy.inexact):
+    dtype = np.dtype(dtype)
+    if issubclass(dtype.type, np.inexact):
         if what == "min":
             return "MY_INFINITY"
         elif what == "max":
@@ -334,9 +356,9 @@ def get_minmax_neutral(what, dtype):
             raise ValueError("what is not min or max.")
     else:
         if what == "min":
-            return str(numpy.iinfo(dtype).max)
+            return str(np.iinfo(dtype).max)
         elif what == "max":
-            return str(numpy.iinfo(dtype).min)
+            return str(np.iinfo(dtype).min)
         else:
             raise ValueError("what is not min or max.")
 
@@ -345,9 +367,9 @@ def get_minmax_neutral(what, dtype):
 
 @context_dependent_memoize
 def get_minmax_kernel(what, dtype):
-    if dtype == numpy.float64:
+    if dtype == np.float64:
         reduce_expr = "f%s(a,b)" % what
-    elif dtype == numpy.float32:
+    elif dtype == np.float32:
         reduce_expr = "f%sf(a,b)" % what
     elif dtype.kind in "iu":
         reduce_expr = "%s(a,b)" % what
@@ -366,9 +388,9 @@ def get_minmax_kernel(what, dtype):
 
 @context_dependent_memoize
 def get_subset_minmax_kernel(what, dtype):
-    if dtype == numpy.float64:
+    if dtype == np.float64:
         reduce_expr = "f%s(a,b)" % what
-    elif dtype == numpy.float32:
+    elif dtype == np.float32:
         reduce_expr = "f%sf(a,b)" % what
     elif dtype.kind in "iu":
         reduce_expr = "%s(a,b)" % what
